########################### PSQF 6270 Example 5a using R #################################

# Set width of output and number of digits to be printed
options("width"=140, "digits"=8)
# Turn off stars for different alpha levels
options(show.signif.stars=FALSE)

# Use this to turn off scientific notation: 
# "fixed notation will be preferred unless it is more than 'scipen' digits wide"
options(scipen=9)

#####  Check to see if packages are downloaded, install if not, then load  #####

if (!require("haven")) install.packages("haven")
library(haven) # To import SAS data with labels as table

if (!require("expss")) install.packages("expss")
library(expss) # To add variable and value labels, sorting

if (!require("psych")) install.packages("psych")
library(psych) # To add descriptive summary functions

if (!require("multcomp")) install.packages("multcomp")
library(multcomp) # To do glht linear combinations in GLMs

if (!require("margins")) install.packages("margins")
library(margins) # To get predicted values like Stata does

if (!require("nlme")) install.packages("nlme")
library(nlme) # To fit multivariate models without random intercepts

if (!require("emmeans")) install.packages("emmeans")
library(emmeans) # To get model-implied means in gls

if (!require("lavaan")) install.packages("lavaan")
library(lavaan) # To fit path models 

if (!require("TeachingDemos")) install.packages("TeachingDemos")
library(TeachingDemos) # To create text output files


######################################################################################
#####                  BEGIN DATA MANIPULATION OF EXAMPLE 5a                     #####
#####                   CHANGE "filesave" TO YOUR FOLDER PATH                    #####
######################################################################################

# Define variables for working directory and data name
filesave = "C:\\Dropbox/22_PSQF6270/PSQF6270_Example5a/"
filename = "PSQF6270_Example5aWide.sas7bdat"
setwd(dir=filesave)

# Import Example 5a SAS data
Example5a_wide = read_sas(data_file=paste0(filesave,filename)) 
# Convert to data frame without labels to use for analysis
Example5a_wide = as.data.frame(Example5a_wide)

# Stack into long format (one row per outcome per family)
Example5a = reshape(Example5a_wide,  direction="long", idvar="FamilyID",
                    varying=c("KidMarital","MomMarital","DadMarital"),
                    v.names="marital", timevar="DVnum", times=c(1,2,3))
# Work-around to add value labels for categorical variables DV
# Make a concatenated list of labels in order of values to be labeled
DVLabels = c("1.Kid", "2.Mom", "3.Dad")
# Create new text-format string variable with the labels instead of values
Example5a$DV = DVLabels[Example5a$DVnum]
# Create per-person dummy codes
Example5a$kid=0
Example5a$mom=0
Example5a$dad=0
Example5a$kid[which(Example5a$DVnum==1)]=1
Example5a$mom[which(Example5a$DVnum==2)]=1
Example5a$dad[which(Example5a$DVnum==3)]=1
# Make new variable for DV with reference=Dad to match other programs
Example5a$DV3=relevel(factor(Example5a$DV), ref=3)

# Remove missing predictors or row-specific outcome (will happen anyway)
Example5a = Example5a[complete.cases(Example5a[ ,
                      c("KidBoy","KidEd12","MomEd12","DadEd12","marital")]),]


######################################################################################
#####                   BEGIN EXAMPLE 5a PART 1 MODELS                           #####
######################################################################################

# Open external file to save results to
txtStart(file=paste0(filesave,"PSQF6270_Example5a_R_Output_Part1mixed.txt"))

####### 4 Versions of Baseline Model 0 #######
print("R Empty Means, Unstructured Variance Models for Marital Attitudes")

print("R Model 0a: General Intercept (Dad=Ref DV) using 2 Dummy Codes")
Model0a = gls(data=Example5a, method="REML", 
              model=marital~1+kid+mom,    # Fixed intercept will be for dad (as omitted)
              correlation=corSymm(form=~DVnum|FamilyID),  # Unstructured correlations
              weights=varIdent(form=~1|DV))               # Separate variance by DV  
print("Print -2LL and Results")  # Btw, AIC and BIC are incorrect (match STATA)
-2*logLik(Model0a); summary(Model0a); summary(Model0a)$sigma 
print("Show R and RCORR matrices for first family in the data") 
getVarCov(Model0a, individual="3996");  
corMatrix(Model0a$modelStruct$corStruct)[[3]] # 3=Dimensions of R here

print("DF=2 Intercept Diff -- Get error that it used Chi-Square instead of F")
F0a = glht(model=Model0a, linfct=rbind(c(0,1,0),c(0,0,1)), df=139)
SaveF0a = summary(F0a, test=Ftest()); SaveF0a # Joint F-test
print("Get and show hidden results for F, dfnum, dfden, and p-value")
SaveF0a$test$fstat; SaveF0a$test$df; SaveF0a$df
pf(SaveF0a$test$fstat,df1=SaveF0a$test$df,df2=SaveF0a$df,lower.tail=FALSE)

print("Missing Intercepts and Difference -- Had to give it correct Denominator DF")
summary(glht(model=Model0a, df=139, linfct=rbind(
  "Kid Intercept (Dad+Diff)"    = c(1,1,0),  # in order of fixed effects
  "Mom Intercept (Dad+Diff)"    = c(1,0,1),
  "Kid vs. Mom: Intercept Diff" = c(0,-1,1))), test=adjusted("none"))

txtComment(" ") # insert blank space in output 


print("R Model 0b: General Intercept (Dad=Ref DV) using Program-Categorical DV")
Model0b = gls(data=Example5a, method="REML", 
              model=marital~1+factor(DV3),  # Fixed intercept will be for DV=3
              correlation=corSymm(form=~DVnum|FamilyID),
              weights=varIdent(form=~1|DV))
print("Print -2LL and Results")  # Btw, AIC and BIC are incorrect (match STATA)
-2*logLik(Model0b); summary(Model0b) 
print("Show R and RCORR matrices for first family in the data") 
getVarCov(Model0b, individual="3996");  
corMatrix(Model0b$modelStruct$corStruct)[[3]] # 3=Dimensions of R here
print("DV means, pairwise mean differences, and omnibus F-test")
print("Error message says appx-satterthwaite used instead -- close but not quite")
emmeans(ref_grid(Model0b), pairwise~DV3, adjust="none"); joint_tests(Model0b)

txtComment(" ") # insert blank space in output 


print("R Model 0c: DV-Specific Intercepts using All 3 Dummy Codes")
Model0c = gls(data=Example5a, method="REML", 
              model=marital~0+kid+mom+dad,                # 0 removes fixed intercept
              correlation=corSymm(form=~DVnum|FamilyID),  # Unstructured correlations
              weights=varIdent(form=~1|DV))               # Separate variance by DV  
print("Print -2LL and Results")  # Btw, AIC and BIC are incorrect (match STATA)
-2*logLik(Model0c); summary(Model0c) 
print("Show R and RCORR matrices for first family in the data") 
getVarCov(Model0c, individual="3996");  
corMatrix(Model0c$modelStruct$corStruct)[[3]] # 3=Dimensions of R here

print("DF=2 Intercept Diff -- Get error that it used Chi-Square instead of F")
F0c = glht(model=Model0c, linfct=rbind(c(-1,1,0),c(0,-1,1)), df=139)
SaveF0c = summary(F0c, test=Ftest()); SaveF0a  # Joint F-test
print("Get and show hidden results for F, dfnum, dfden, and p-value")
SaveF0c$test$fstat; SaveF0c$test$df; SaveF0c$df
pf(SaveF0c$test$fstat,df1=SaveF0c$test$df,df2=SaveF0c$df,lower.tail=FALSE)

print("Pairwise Intercept Diffs -- Had to give it correct Denominator DF")
summary(glht(model=Model0c, df=139, linfct=rbind(
  "Kid vs. Mom: Intercept Diff" = c(-1,1,0),  # in order of fixed effects
  "Kid vs. Dad: Intercept Diff" = c(-1,0,1),
  "Mom vs. Dad: Intercept Diff" = c(0,-1,1))), test=adjusted("none"))

txtComment(" ") # insert blank space in output 


print("R Model 0d: DV-Specific Intercepts using Program-Categorical DV")
Model0d = gls(data=Example5a, method="REML", 
              model=marital~0+factor(DV),  # 0 removes fixed intercept
              correlation=corSymm(form=~DVnum|FamilyID),
              weights=varIdent(form=~1|DV))
print("Print -2LL and Results")  # Btw, AIC and BIC are incorrect (match STATA)
-2*logLik(Model0d); summary(Model0d) 
print("Show R and RCORR matrices for first family in the data") 
getVarCov(Model0d, individual="3996");  
corMatrix(Model0d$modelStruct$corStruct)[[3]] # 3=Dimensions of R here
print("DV means, pairwise mean differences, and omnibus F-test")
print("Error message says appx-satterthwaite used instead -- close but not quite")
emmeans(ref_grid(Model0d), pairwise~DV, adjust="none"); joint_tests(Model0d)

txtComment(" ") # insert blank space in output 


print("R Model 1: DV-Specific Intercepts -- Add Kid Gender")
Model1 = gls(data=Example5a, method="REML", 
             model=marital~0+kid+mom+dad+kid:KidBoy+mom:KidBoy+dad:KidBoy,
             correlation=corSymm(form=~DVnum|FamilyID),  # Unstructured correlations
             weights=varIdent(form=~1|DV))               # Separate variance by DV  
print("Print -2LL and Results")  # Btw, AIC and BIC are incorrect (match STATA)
-2*logLik(Model1); summary(Model1) 
print("Show R and RCORR matrices for first family in the data") 
getVarCov(Model1, individual="3996");  
corMatrix(Model1$modelStruct$corStruct)[[3]] # 3=Dimensions of R here

print("DF=2 Diff in KidBoy Effect -- Get error that it used Chi-Square instead of F")
F1 = glht(model=Model1, linfct=rbind(c(0,0,0,-1,1,0),c(0,0,0,-1,0,1)), df=138)
SaveF1 = summary(F1, test=Ftest()); SaveF0a  # Joint F-test
print("Get and show hidden results for F, dfnum, dfden, and p-value")
SaveF1$test$fstat; SaveF1$test$df; SaveF1$df
pf(SaveF1$test$fstat,df1=SaveF1$test$df,df2=SaveF1$df,lower.tail=FALSE)

print("KidBoy Slope Diffs -- Had to give it correct Denominator DF")
summary(glht(model=Model1, df=138, linfct=rbind(
  "Kid vs. Mom: KidBoy Slope Diff"     = c(0,0,0,-1,1,0),  # in order of fixed effects
  "Kid vs. Dad: KidBoy Slope Diff"     = c(0,0,0,-1,0,1),
  "Mom vs. Dad: KidBoy Slope Diff"     = c(0,0,0,0,-1,1),
  "Parent KidBoy Effect"               = c(0,0,0,0,1/2,1/2),
  "Kids vs. Parent KidBoy Effect Diff" = c(0,0,0,-1,1/2,1/2))), test=adjusted("none"))

txtComment(" ") # insert blank space in output 


print("R Model 2: DV-Specific Intercepts -- KidBoy on Kid Only, Add Own Educ")
Model2 = gls(data=Example5a, method="REML", 
             model=marital~0+kid+mom+dad +kid:KidBoy +kid:KidEd12+mom:MomEd12+dad:DadEd12,
             correlation=corSymm(form=~DVnum|FamilyID),  # Unstructured correlations
             weights=varIdent(form=~1|DV))               # Separate variance by DV  
print("Print -2LL and Results")  # Btw, AIC and BIC are incorrect (match STATA)
-2*logLik(Model2); summary(Model2) 
print("Show R and RCORR matrices for first family in the data") 
getVarCov(Model2, individual="3996");  
corMatrix(Model2$modelStruct$corStruct)[[3]] # 3=Dimensions of R here

txtComment(" ") # insert blank space in output


print("R Model 3: DV-Specific Intercepts -- Add Dad Educ (Control for Own Educ)")
Model3 = gls(data=Example5a, method="REML", 
             model=marital~0+kid+mom+dad+ kid:KidBoy +kid:KidEd12+mom:MomEd12+dad:DadEd12
                           +kid:DadEd12+mom:DadEd12,
             correlation=corSymm(form=~DVnum|FamilyID),  # Unstructured correlations
             weights=varIdent(form=~1|DV))               # Separate variance by DV  
print("Print -2LL and Results")  # Btw, AIC and BIC are incorrect (match STATA)
-2*logLik(Model3); summary(Model3) 
print("Show R and RCORR matrices for first family in the data") 
getVarCov(Model3, individual="3996");  
corMatrix(Model3$modelStruct$corStruct)[[3]] # 3=Dimensions of R here

print("DadEd Slope Diffs -- Had to give it correct Denominator DF")
summary(glht(model=Model3, df=136, linfct=rbind(
  "Kid vs. Mom: DadEd12 Slope Diff" = c(0,0,0,0,0,0,0,-1,1),  # in order of fixed effects
  "Kid vs. Dad: DadEd12 Slope Diff" = c(0,0,0,0,0,0,1,-1,0),
  "Mom vs. Dad: DadEd12 Slope Diff" = c(0,0,0,0,0,0,1,0,-1))), test=adjusted("none"))

print("Save yhat from fixed effects and Pearson residuals")
Example5a$Model3pred = predict(Model3, type="response")
Example5a$Model3res = residuals(Model3, type="pearson")
print ("Histogram of Residuals for normality")
hist(x=Example5a$Model3res, freq=FALSE, ylab="Density",xlab="Model 3 Residuals") 
print ("Scatterplot of residuals by prediced for constant variance")
plot(x=Example5a$Model3res, y=Example5a$Model3pred,
     ylab="Residual",xlab="Model 3 Predicted Outcome") 

txtComment(" ") # insert blank space in output 

# Close external output file
txtStop()


# Open external file to save results to
txtStart(file=paste0(filesave,"PSQF6270_Example5a_R_Output_Part2mixed.txt"))

print("R Model 4: DV-Specific Intercepts -- Add Mom Educ (Controlling for Own+Dad Educ)")
print("To match path model, switch to ML estimation, but robust SEs not directly available")
Model4 = gls(data=Example5a, method="ML", 
             model=marital~0+kid+mom+dad+ kid:KidBoy +kid:KidEd12+mom:MomEd12+dad:DadEd12
             +kid:DadEd12+mom:DadEd12 +kid:MomEd12+dad:MomEd12,
             correlation=corSymm(form=~DVnum|FamilyID),  # Unstructured correlations
             weights=varIdent(form=~1|DV))               # Separate variance by DV  
print("Print -2LL and Results")  # Btw, AIC and BIC are incorrect (match STATA)
-2*logLik(Model4); summary(Model4) 
print("Show R and RCORR matrices for first family in the data") 
getVarCov(Model4, individual="3996");  
corMatrix(Model4$modelStruct$corStruct)[[3]] # 3=Dimensions of R here

print("DadEd Slope Diffs -- Had to give it correct Denominator DF")
summary(glht(model=Model4, df=135, linfct=rbind(
  "Kid vs. Mom: DadEd12 Slope Diff" = c(0,0,0,0,0,0,0,-1,1,0,0),  # in order of fixed effects
  "Kid vs. Dad: DadEd12 Slope Diff" = c(0,0,0,0,0,0,1,-1,0,0,0),
  "Mom vs. Dad: DadEd12 Slope Diff" = c(0,0,0,0,0,0,1,0,-1,0,0),
  "Kid vs. Mom: MomEd12 Slope Diff" = c(0,0,0,0,0,1,0,0,0,-1,0),  
  "Kid vs. Dad: MomEd12 Slope Diff" = c(0,0,0,0,0,0,0,0,0,-1,1),
  "Mom vs. Dad: MomEd12 Slope Diff" = c(0,0,0,0,0,-1,0,0,0,0,1))), test=adjusted("none"))

print("Save predicted marital attitudes and correlate with actual marital attitudes") 
Example5a$Pred = predict(Model4, type="response")

rPred1 = cor.test(x=Example5a$Pred[which(Example5a$DVnum==1)], 
                  y=Example5a$marital[which(Example5a$DVnum==1)], method="pearson")
print("R and R2 for DV=1 Kid"); rPred1$estimate; rPred1$estimate^2

rPred2 = cor.test(x=Example5a$Pred[which(Example5a$DVnum==2)], 
                  y=Example5a$marital[which(Example5a$DVnum==2)], method="pearson")
print("R and R2 for DV=2 Mom"); rPred2$estimate; rPred2$estimate^2

rPred3 = cor.test(x=Example5a$Pred[which(Example5a$DVnum==3)], 
                  y=Example5a$marital[which(Example5a$DVnum==3)], method="pearson")
print("R and R2 for DV=3 Dad"); rPred3$estimate; rPred3$estimate^2


# Close external output file
txtStop()


######################################################################################
#####                   BEGIN EXAMPLE 5a PART 2 MODELS                           #####
######################################################################################

# Example of how to export a .csv file for use in Mplus
# Copy data, replace all missing values with -999 for Mplus
Example5a_Mplus = Example5a_wide
Example5a_Mplus[is.na(Example5a_Mplus)] <- -999
# Write to .csv file without column names
write.table(x=Example5a_Mplus, col.names=FALSE, row.names=FALSE, sep=",",
            file=paste0(filesave,"PSQF6270_Example5aWide_R.csv"))
          
# Open external file to save results to
txtStart(file=paste0(filesave,"PSQF6270_Example5a_R_Output_Part2path.txt"))

print("R Model 3: Own Education + Dad Education a Predictor of Each Attitude")

# Create model syntax as separate text object
Syntax3 = "
# Residual variances estimated separately (by default)
  KidMarital ~~ KidMarital; MomMarital ~~ MomMarital; DadMarital ~~ DadMarital
# All possible pairwise residual covariances (not estimated by default)
  KidMarital ~~ MomMarital + DadMarital; MomMarital ~~ DadMarital
# All intercepts estimated separately (by default)
  KidMarital ~ 1; MomMarital ~ 1; DadMarital ~ 1
# Regressions: y outcomes ON x predictors (label to do math on later)  
  KidMarital ~ KidBoy + KidEd12
  MomMarital ~ MomEd12
  KidMarital ~ (DadEd2K)*DadEd12 
  MomMarital ~ (DadEd2M)*DadEd12 
  DadMarital ~ (DadEd2D)*DadEd12
# Getting differences in effect of DadEd for each person
  KvMDadEd := DadEd2M - DadEd2K;  # Kid v. Mom: Dad Educ Effect Diff
  KvDDadEd := DadEd2D - DadEd2K;  # Kid v. Dad: Dad Educ Effect Diff
  MvDDadEd := DadEd2D - DadEd2M;  # Mom v. Dad: Dad Educ Effect Diff
"
print("lavaan path analysis model estimated with ML on wide-format data")
PathModel3 = lavaan(data=Example5a_wide, model=Syntax3, estimator="MLR", mimic="mplus")
summary(PathModel3, fit.measures=TRUE, rsquare=TRUE, standardized=TRUE, ci=TRUE)
print("Request sorted modification indices for p<.05 to troubleshoot local misfit")
modindices(object=PathModel3, sort=TRUE, minimum.value=3.84)
print("Request residual covariance matrix =leftover from observed minus predicted")
resid(object=PathModel3, type="raw") # also type="cor" for correlation matrix

txtComment(" ") # insert blank space in output 


print("R Model 4: Own + Dad + Mom Education a Predictor of Each Attitude")

# Create model syntax as separate text object
Syntax4 = "
# Residual variances estimated separately (by default)
  KidMarital ~~ KidMarital; MomMarital ~~ MomMarital; DadMarital ~~ DadMarital
# All possible pairwise residual covariances (not estimated by default)
  KidMarital ~~ MomMarital + DadMarital; MomMarital ~~ DadMarital
# All intercepts estimated separately (by default)
  KidMarital ~ 1; MomMarital ~ 1; DadMarital ~ 1
# Regressions: y outcomes ON x predictors (label to do math on later)  
  KidMarital ~ KidBoy + KidEd12
  KidMarital ~ (DadEd2K)*DadEd12 
  MomMarital ~ (DadEd2M)*DadEd12 
  DadMarital ~ (DadEd2D)*DadEd12
# New effects here
  KidMarital ~ (MomEd2K)*MomEd12 
  MomMarital ~ (MomEd2M)*MomEd12 
  DadMarital ~ (MomEd2D)*MomEd12
  
# Getting differences in effect of DadEd for each person
  KvMDadEd := DadEd2M - DadEd2K;  # Kid v. Mom: Dad Educ Effect Diff
  KvDDadEd := DadEd2D - DadEd2K;  # Kid v. Dad: Dad Educ Effect Diff
  MvDDadEd := DadEd2D - DadEd2M;  # Mom v. Dad: Dad Educ Effect Diff
  KvMMomEd := MomEd2M - MomEd2K;  # Kid v. Mom: Mom Educ Effect Diff
  KvDMomEd := MomEd2D - MomEd2K;  # Kid v. Dad: Mom Educ Effect Diff
  MvDMomEd := MomEd2D - MomEd2M;  # Mom v. Dad: Mom Educ Effect Diff
"
print("lavaan path analysis model estimated with ML on wide-format data")
PathModel4 = lavaan(data=Example5a_wide, model=Syntax4, estimator="MLR", mimic="mplus")
summary(PathModel4, fit.measures=TRUE, rsquare=TRUE, standardized=TRUE, ci=TRUE)
print("Request sorted modification indices for p<.05 to troubleshoot local misfit")
modindices(object=PathModel4, sort=TRUE, minimum.value=3.84)
print("Request residual correlation matrix =leftover from observed minus predicted")
resid(object=PathModel4, type="raw") # also type="cor" for correlation matrix


# Close external output file
txtStop()

